
#ifndef __QDIGITSIGN_H
#define __QDIGITSIGN_H

#ifndef WIN32
#define QDIGITSIGN_API
#else
#ifdef QDIGITSIGN_STATIC
#define QDIGITSIGN_API
#else
#ifdef QDIGITSIGN_EXPORTS
#define QDIGITSIGN_API __declspec(dllexport)
#else
#define QDIGITSIGN_API __declspec(dllimport)
#endif
#endif
#endif

#define MAX_LEN							256

#include "definitions.h"

#define VERIFIED_CERT_VALIDITY			0x000001
#define VERIFIED_CERT_REVOKED			0x000002
#define VERIFIED_CERT_SUSPENDED			0x000004
#define VERIFIED_CERT_GOOD				0x000008
#define VERIFIED_CERT_SHA256			0x000010
#define VERIFIED_CERT_QUALIFIED			0x000020
#define VERIFIED_CERT_SIGNATURE			0x000040

#define VERIFIED_CRL_LOADED				0x000080
#define VERIFIED_CRL_EXPIRED			0x000100

#define VERIFIED_SIGNED_ATTRIBUTE_CT	0x000200
#define VERIFIED_SIGNED_ATTRIBUTE_MD    0x000400
#define VERIFIED_SIGNED_ATTRIBUTE_SC    0x000800

#define VERIFIED_SIGNATURE				0x001000
#define VERIFIED_SHA256					0x002000

#define VERIFIED_CACERT_VALIDITY		0x004000
#define VERIFIED_CACERT_REVOKED			0x008000
#define VERIFIED_CACERT_SUSPENDED		0x010000
#define VERIFIED_CACERT_GOOD			0x020000
#define VERIFIED_CACERT_SIGNATURE		0x040000
#define VERIFIED_CACRL_LOADED			0x080000
#define VERIFIED_CACERT_FOUND			0x100000

#define VERIFIED_KEY_USAGE				0x200000

// TODO
// certificate store con cache degli esiti delle verifiche dei certificati
// stato revoca e validit dei certificati nella chain


#define REVOCATION_STATUS_GOOD  0
#define REVOCATION_STATUS_REVOKED  1
#define REVOCATION_STATUS_SUSPENDED 2
#define REVOCATION_STATUS_UNKNOWN  3
#define REVOCATION_STATUS_NOTLOADED  4

#define QDIGITSIGN_OPT_PKCS11				1
#define QDIGITSIGN_OPT_SLOT					2
#define QDIGITSIGN_OPT_PIN					3
#define QDIGITSIGN_OPT_ALIAS				4
#define QDIGITSIGN_OPT_CADES				5
#define QDIGITSIGN_OPT_XADES				5
#define QDIGITSIGN_OPT_DETACHED				6
#define QDIGITSIGN_OPT_INPUTFILE			7
#define QDIGITSIGN_OPT_OUTPUTFILE			8
#define QDIGITSIGN_OPT_INPUTFILE_TYPE		9
#define QDIGITSIGN_OPT_TSA_URL				10
#define QDIGITSIGN_OPT_TSA_USERNAME			11
#define QDIGITSIGN_OPT_TSA_PASSWORD			12
#define QDIGITSIGN_OPT_VERIFY_REVOCATION	13
#define QDIGITSIGN_OPT_LOG_LEVEL			14
#define QDIGITSIGN_OPT_LOG_FILE				15
#define QDIGITSIGN_OPT_INPUTFILE_PLAINTEXT	16
#define QDIGITSIGN_OPT_CACERT_DIR			17
#define QDIGITSIGN_OPT_PDF_SUBFILTER		18
#define QDIGITSIGN_OPT_CONFIG_FILE			19
#define QDIGITSIGN_OPT_PROXY				20
#define QDIGITSIGN_OPT_PROXY_PORT			21
#define QDIGITSIGN_OPT_PROXY_USRPASS		22
#define QDIGITSIGN_OPT_OID_MAP_FILE			23
#define QDIGITSIGN_OPT_TCP_TIMEOUT			24
#define QDIGITSIGN_OPT_PDF_REASON			25
#define QDIGITSIGN_OPT_PDF_NAME				26
#define QDIGITSIGN_OPT_PDF_LOCATION			27
#define QDIGITSIGN_OPT_PDF_PAGE				28
#define QDIGITSIGN_OPT_PDF_LEFT				29
#define QDIGITSIGN_OPT_PDF_BOTTOM			30
#define QDIGITSIGN_OPT_PDF_WIDTH			31
#define QDIGITSIGN_OPT_PDF_HEIGHT			32
#define QDIGITSIGN_OPT_PDF_IMAGEPATH			33
#define QDIGITSIGN_OPT_PDF_GRAPHOMETRIC_DATA	34
#define QDIGITSIGN_OPT_PDF_GRAPHOMETRIC_DATA_VER 35

#define QDIGITSIGN_OPT_ATR_LIST_FILE		36
#define QDIGITSIGN_OPT_HASH_ALGO			37

#define QDIGITSIGN_OPT_LICENSEE				38
#define QDIGITSIGN_OPT_PRODUCTKEY			39

#define QDIGITSIGN_OPT_RS_OTP_PIN			40
#define QDIGITSIGN_OPT_RS_HSMTYPE			41
#define QDIGITSIGN_OPT_RS_TYPE_OTP_AUTH		42
#define QDIGITSIGN_OPT_RS_USERNAME			43
#define QDIGITSIGN_OPT_RS_PASSWORD			44
#define QDIGITSIGN_OPT_RS_CERTID			45
#define QDIGITSIGN_OPT_RS_SERVICE_URL		46


#define QDIGITSIGN_OPT_PDF_DESCRIPTION		50
#define QDIGITSIGN_OPT_PDF_NAME_LABEL		51
#define QDIGITSIGN_OPT_PDF_REASON_LABEL		52
#define QDIGITSIGN_OPT_PDF_LOCATION_LABEL	53


#define QDIGITSIGN_PDF_SUBFILTER_PKCS_DETACHED "adbe.pkcs7.detached"
#define QDIGITSIGN_PDF_SUBFILTER_ETSI_CADES "ETSI.CAdES.detached"


#define QDIGITSIGN_ERROR_BASE			0x84000000


#define QDIGITSIGN_ERROR_UNEXPECTED			QDIGITSIGN_ERROR_BASE + 1
#define QDIGITSIGN_ERROR_FILE_NOT_FOUND		QDIGITSIGN_ERROR_BASE + 2
#define QDIGITSIGN_ERROR_DETACHED_PKCS7		QDIGITSIGN_ERROR_BASE + 3
#define QDIGITSIGN_ERROR_CERT_REVOKED		QDIGITSIGN_ERROR_BASE + 4
#define QDIGITSIGN_ERROR_INVALID_FILE		QDIGITSIGN_ERROR_BASE + 5
#define QDIGITSIGN_ERROR_INVALID_P11		QDIGITSIGN_ERROR_BASE + 6
#define QDIGITSIGN_ERROR_INVALID_ALIAS		QDIGITSIGN_ERROR_BASE + 7
#define QDIGITSIGN_ERROR_INVALID_SIGOPT		QDIGITSIGN_ERROR_BASE + 8
#define QDIGITSIGN_ERROR_ARRS_BASE			QDIGITSIGN_ERROR_BASE + 0x00100000
#define QDIGITSIGN_ERROR_CERT_INVALID		QDIGITSIGN_ERROR_BASE + 9
#define QDIGITSIGN_ERROR_CERT_EXPIRED		QDIGITSIGN_ERROR_BASE + 10
#define QDIGITSIGN_ERROR_CACERT_NOTFOUND	QDIGITSIGN_ERROR_BASE + 11
#define QDIGITSIGN_ERROR_CERT_NOTFOUND		QDIGITSIGN_ERROR_BASE + 12

#define QDIGITSIGN_FILETYPE_PLAINTEXT	0
#define QDIGITSIGN_FILETYPE_P7M			1
#define QDIGITSIGN_FILETYPE_PDF			2
#define QDIGITSIGN_FILETYPE_M7M			3
#define QDIGITSIGN_FILETYPE_TSR			4
#define QDIGITSIGN_FILETYPE_TST			5
#define QDIGITSIGN_FILETYPE_TSD			6
#define QDIGITSIGN_FILETYPE_XML			7
#define QDIGITSIGN_FILETYPE_AUTO		8

/*
#define QDIGITSIGN_VERIFYTYPE_P7M		1
#define QDIGITSIGN_VERIFYTYPE_TSD		2
#define QDIGITSIGN_VERIFYTYPE_TST		3
#define QDIGITSIGN_VERIFYTYPE_TSR		4
#define QDIGITSIGN_VERIFYTYPE_PDF		5
#define QDIGITSIGN_VERIFYTYPE_XML		6
#define QDIGITSIGN_VERIFYTYPE_M7M        7
*/
#define QDIGITSIGN_ALGO_SHA1			1
#define QDIGITSIGN_ALGO_SHA256			2
#define QDIGITSIGN_ALGO_SHA512			3
#define QDIGITSIGN_ALGO_MD5				4

#define LOG_TYPE_ERROR			1
#define LOG_TYPE_WARNING		2
#define LOG_TYPE_MESSAGE		3
#define LOG_TYPE_DEBUG			4  

#define TYPE_OCSP 1
#define TYPE_CRL 2



typedef struct _REVOCATION_INFO
{
	int nType;  // OCSP, CRL
	char szExpiration[40];
	char szThisUpdate[40];
	int nRevocationStatus;
	char szRevocationDate[40];
} REVOCATION_INFO;


typedef struct _SIGNER_INFO
{
	char szCN[MAX_LEN * 2];
	char szSN[MAX_LEN * 2];
	char szCACN[MAX_LEN * 2];
	char** pszExtensions;
	int nExtensionsCount;
	char szExpiration[MAX_LEN];
	char szValidFrom[MAX_LEN];
	long bitmask;
	char szDigestAlgorithm[MAX_LEN];
	char szSigningTime[MAX_LEN];
	char szCertificateV2[MAX_LEN];
	BOOL b2011Error;
	BYTE* pCertificate;
	int nCertLen;
	void* pTimeStamp;
	REVOCATION_INFO* pRevocationInfo;
	void* pCounterSignatures;
	int nCounterSignatureCount;
} SIGNER_INFO;

typedef struct _TS_INFO
{
	SIGNER_INFO signerInfo;
	char szTimestamp[MAX_LEN];  
	char szTimeStampImprintAlgorithm[MAX_LEN];
	char szTimeStampMessageImprint[MAX_LEN];
	char szTimeStampSerial[MAX_LEN];
} TS_INFO;
 

typedef struct _SIGNER_INFOS 
{
	SIGNER_INFO* pSignerInfo;
	int nCount;
} SIGNER_INFOS;

typedef struct _VERIFY_INFO
{
	SIGNER_INFOS* pSignerInfos;
	TS_INFO* pTSInfo;

} VERIFY_INFO;

typedef struct _VERIFY_RESULT
{
	int nResultType;
	BOOL bVerifyCRL;
	VERIFY_INFO verifyInfo;
	long nErrorCode;
	char szInputFile[MAX_PATH];
	char szPlainTextFile[MAX_PATH];
} VERIFY_RESULT;


typedef struct _CERTIFICATE
{
	char szLabel[MAX_LEN * 2];
	char szCN[MAX_LEN * 2];
	char szSN[MAX_LEN * 2];
	char szCACN[MAX_LEN * 2];
	char szExpiration[MAX_LEN];
	char szValidFrom[MAX_LEN];
	BYTE* pCertificate;
	int nCertLen;
} CERTIFICATE;


typedef struct _CERTIFICATES 
{
	CERTIFICATE* pCertificate;
	int nCount;
} CERTIFICATES;


typedef void* QDIGITSIGN_CTX;

#ifdef  __cplusplus
extern "C" {
#endif

// Imposta le opzioni generali della libreria
QDIGITSIGN_API long qdigitsign_set(int option, void* value);

QDIGITSIGN_API long qdigitsign_set_int(int option, int value);

QDIGITSIGN_API long qdigitsign_set_string(int option, char* value);

// libera la memoria allocata dalla libreria
QDIGITSIGN_API void qdigitsign_cleanup();

///////////////////////////////////
// Funzioni di firma digitale

// inizializza la funzione di firma
QDIGITSIGN_API QDIGITSIGN_CTX qdigitsign_sign_init(void);

// imposta le opzioni di firma
QDIGITSIGN_API long qdigitsign_sign_set_int(QDIGITSIGN_CTX ctx, int option, int value);

QDIGITSIGN_API long qdigitsign_sign_set_string(QDIGITSIGN_CTX ctx, int option, char* value);

QDIGITSIGN_API long qdigitsign_sign_set(QDIGITSIGN_CTX ctx, int option, void* value);

// esegue la firma secondo le opzioni passate
QDIGITSIGN_API long qdigitsign_sign_sign(QDIGITSIGN_CTX ctx);

// libera la memoria allocata nell'operazione di firma
QDIGITSIGN_API long qdigitsign_sign_cleanup(QDIGITSIGN_CTX ctx);

QDIGITSIGN_API long qdigitsign_sign_getcertificates(QDIGITSIGN_CTX ctx, CERTIFICATES* certs);

QDIGITSIGN_API void qdigitsign_sign_freecertificates(CERTIFICATES* certs);

//
///////////////////////////////////////////

//////////////////////////////////////////
// Funzioni di verifica della firma

// inizializza l'operazione di verifica
QDIGITSIGN_API QDIGITSIGN_CTX qdigitsign_verify_init(void);

// imposta le opzioni di verifica
QDIGITSIGN_API long qdigitsign_verify_set(QDIGITSIGN_CTX ctx, int option, void* value);
QDIGITSIGN_API long qdigitsign_verify_set_int(QDIGITSIGN_CTX ctx, int option, int value);
QDIGITSIGN_API long qdigitsign_verify_set_string(QDIGITSIGN_CTX ctx, int option, char* value);

// esegue l'operazione di verifica secondo le opzioni passate
QDIGITSIGN_API long qdigitsign_verify_verify(QDIGITSIGN_CTX ctx, VERIFY_RESULT* pVerifyResult);

// libera la memoria allocata per la struttura dei risultati delle verifica
QDIGITSIGN_API long qdigitsign_verify_cleanup_result(VERIFY_RESULT* pVerifyResult);

// libera la memoria allocata nell'operazione di verifica
QDIGITSIGN_API long qdigitsign_verify_cleanup(QDIGITSIGN_CTX ctx);
// operazione di sbustamento veloce

// inizializza l'operazione di verifica
QDIGITSIGN_API QDIGITSIGN_CTX qdigitsign_read_init(void);

// imposta le opzioni di verifica
QDIGITSIGN_API long qdigitsign_read_set(QDIGITSIGN_CTX ctx, int option, void* value);
QDIGITSIGN_API long qdigitsign_read_set_int(QDIGITSIGN_CTX ctx, int option, int value);
QDIGITSIGN_API long qdigitsign_read_set_string(QDIGITSIGN_CTX ctx, int option, char* value);

// esegue l'operazione di verifica secondo le opzioni passate
QDIGITSIGN_API long qdigitsign_read_read(QDIGITSIGN_CTX ctx);

// libera la memoria allocata nell'operazione di verifica
QDIGITSIGN_API long qdigitsign_read_cleanup(QDIGITSIGN_CTX ctx);

#ifdef  __cplusplus
}
#endif

#endif // __QDIGITSIGN_H

